﻿
using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;

BenchmarkRunner.Run<VirtualMethodBenchmarks>();

public class A {
	public int m(int x) {
		return x * 10 + 5;
	}

	[System.Runtime.CompilerServices.MethodImpl(System.Runtime.CompilerServices.MethodImplOptions.NoInlining)]
	public int mNonInlined(int x) {
		return x * 10 + 5;
	}

	public virtual int f(int x) {
		return x * 10 + 5;
	}
}

public class B : A {
	public override int f(int x) {
		return x * 10 + 6;
	}
}

public class C : A {
	public override int f(int x) {
		return x * 10 + 7;
	}
}

public class D : A {
	public override int f(int x) {
		return x * 10 + 8;
	}
}

public class VirtualMethodBenchmarks {

	public A _a = new A();

	public B _b = new B();
	public C _c = new C();
	public D _d = new D();

	public A _someInstance = new C();

	[Benchmark]
	public int NonVirtual() {
		return _a.m(3);
	}

	[Benchmark]
	public int NonVirtualNoInlining() {
		return _a.mNonInlined(3);
	}

	[Benchmark]
	public int VirtualAInstance() {
		return _a.f(3);
	}

	[Benchmark]
	public int VirtualCInstance() {
		return _someInstance.f(3);
	}

	private A _aPrivate = new A();
	private A _someInstancePrivate = new C();

	[Benchmark]
	public int VirtualAInstancePrivate() {
		return _aPrivate.f(3);
	}

	[Benchmark]
	public int VirtualCInstancePrivate() {
		return _someInstancePrivate.f(3);
	}
}